@@ -2,25 +2,38 @@
 '''
 A collection of mixins useful for the various *Client interfaces
 '''
-from __future__ import print_function
-from __future__ import absolute_import
-import collections
-import logging
-import traceback
-import multiprocessing
-import weakref
 
+# Import Python libs
+from __future__ import absolute_import, print_function, with_statement, unicode_literals
+import fnmatch
+import signal
+import logging
+import weakref
+import traceback
+import collections
+import copy as pycopy
+
+# Import Salt libs
 import salt.exceptions
 import salt.minion
-import salt.utils
+import salt.utils.args
+import salt.utils.doc
+import salt.utils.error
 import salt.utils.event
 import salt.utils.jid
 import salt.utils.job
+import salt.utils.lazy
+import salt.utils.platform
+import salt.utils.process
+import salt.utils.state
+import salt.utils.user
+import salt.utils.versions
 import salt.transport
-from salt.utils.error import raise_error
-from salt.utils.event import tagify
-from salt.utils.doc import strip_rst as _strip_rst
-from salt.utils.lazy import verify_fun
+import salt.log.setup
+from salt.ext import six
+
+# Import 3rd-party libs
+import tornado.stack_context
 
 log = logging.getLogger(__name__)
 
@@ -35,6 +48,8 @@ CLIENT_INTERNAL_KEYWORDS = frozenset([
     '__jid__',
     '__tag__',
     '__user__',
+    'username',
+    'password'
 ])
 
 
@@ -45,6 +60,12 @@ class ClientFuncsDict(collections.MutableMapping):
     def __init__(self, client):
         self.client = client
 
+    def __getattr__(self, attr):
+        '''
+        Provide access eg. to 'pack'
+        '''
+        return getattr(self.client.functions, attr)
+
     def __setitem__(self, key, val):
         raise NotImplementedError()
 
@@ -65,21 +86,25 @@ class ClientFuncsDict(collections.MutableMapping):
                    'kwargs': kwargs,
                    }
             pub_data = {}
+            # Copy kwargs keys so we can iterate over and pop the pub data
+            kwargs_keys = list(kwargs)
+
             # pull out pub_data if you have it
-            for kwargs_key, kwargs_value in kwargs.items():
+            for kwargs_key in kwargs_keys:
                 if kwargs_key.startswith('__pub_'):
                     pub_data[kwargs_key] = kwargs.pop(kwargs_key)
 
             async_pub = self.client._gen_async_pub(pub_data.get('__pub_jid'))
 
-            user = salt.utils.get_specific_user()
-            return self.client._proc_function(key,
-                   low,
-                   user,
-                   async_pub['tag'],  # TODO: fix
-                   async_pub['jid'],  # TODO: fix
-                   False,  # Don't daemonize
-                   )
+            user = salt.utils.user.get_specific_user()
+            return self.client._proc_function(
+                key,
+                low,
+                user,
+                async_pub['tag'],  # TODO: fix
+                async_pub['jid'],  # TODO: fix
+                False,  # Don't daemonize
+            )
         return wrapper
 
     def __len__(self):
@@ -115,10 +140,10 @@ class SyncClientMixin(object):
         ret = channel.send(load)
         if isinstance(ret, collections.Mapping):
             if 'error' in ret:
-                raise_error(**ret['error'])
+                salt.utils.error.raise_error(**ret['error'])
         return ret
 
-    def cmd_sync(self, low, timeout=None):
+    def cmd_sync(self, low, timeout=None, full_return=False):
         '''
         Execute a runner function synchronously; eauth is respected
 
@@ -134,21 +159,21 @@ class SyncClientMixin(object):
                 'eauth': 'pam',
             })
         '''
-        event = salt.utils.event.get_master_event(self.opts, self.opts['sock_dir'])
+        event = salt.utils.event.get_master_event(self.opts, self.opts['sock_dir'], listen=True)
         job = self.master_call(**low)
         ret_tag = salt.utils.event.tagify('ret', base=job['tag'])
 
         if timeout is None:
-            timeout = 300
-        ret = event.get_event(tag=ret_tag, full=True, wait=timeout)
+            timeout = self.opts.get('rest_timeout', 300)
+        ret = event.get_event(tag=ret_tag, full=True, wait=timeout, auto_reconnect=True)
         if ret is None:
             raise salt.exceptions.SaltClientTimeout(
                 "RunnerClient job '{0}' timed out".format(job['jid']),
                 jid=job['jid'])
 
-        return ret['data']['return']
+        return ret if full_return else ret['data']['return']
 
-    def cmd(self, fun, arg=None, pub_data=None, kwarg=None):
+    def cmd(self, fun, arg=None, pub_data=None, kwarg=None, print_event=True, full_return=False):
         '''
         Execute a function
 
@@ -195,7 +220,9 @@ class SyncClientMixin(object):
             raise salt.exceptions.SaltInvocationError(
                 'kwarg must be formatted as a dictionary'
             )
-        arglist = salt.utils.args.parse_input(arg)
+        arglist = salt.utils.args.parse_input(
+            arg,
+            no_parse=self.opts.get('no_parse', []))
 
         # if you were passed kwarg, add it to arglist
         if kwarg:
@@ -206,9 +233,9 @@ class SyncClientMixin(object):
             self.functions[fun], arglist, pub_data
         )
         low = {'fun': fun,
-               'args': args,
-               'kwargs': kwargs}
-        return self.low(fun, low)
+               'arg': args,
+               'kwarg': kwargs}
+        return self.low(fun, low, print_event=print_event, full_return=full_return)
 
     @property
     def mminion(self):
@@ -216,26 +243,69 @@ class SyncClientMixin(object):
             self._mminion = salt.minion.MasterMinion(self.opts, states=False, rend=False)
         return self._mminion
 
-    def low(self, fun, low):
+    def low(self, fun, low, print_event=True, full_return=False):
+        '''
+        Check for deprecated usage and allow until Salt Fluorine.
+        '''
+        msg = []
+        if 'args' in low:
+            msg.append('call with arg instead')
+            low['arg'] = low.pop('args')
+        if 'kwargs' in low:
+            msg.append('call with kwarg instead')
+            low['kwarg'] = low.pop('kwargs')
+
+        if msg:
+            salt.utils.versions.warn_until('Fluorine', ' '.join(msg))
+
+        return self._low(fun, low, print_event=print_event, full_return=full_return)
+
+    @property
+    def store_job(self):
+        '''
+        Helper that allows us to turn off storing jobs for different classes
+        that may incorporate this mixin.
+        '''
+        try:
+            class_name = self.__class__.__name__.lower()
+        except AttributeError:
+            log.warning(
+                'Unable to determine class name',
+                exc_info_on_loglevel=logging.DEBUG
+            )
+            return True
+
+        try:
+            return self.opts['{0}_returns'.format(class_name)]
+        except KeyError:
+            # No such option, assume this isn't one we care about gating and
+            # just return True.
+            return True
+
+    def _low(self, fun, low, print_event=True, full_return=False):
         '''
         Execute a function from low data
         Low data includes:
             required:
                 - fun: the name of the function to run
             optional:
-                - args: a list of args to pass to fun
-                - kwargs: kwargs for fun
+                - arg: a list of args to pass to fun
+                - kwarg: kwargs for fun
                 - __user__: user who is running the command
                 - __jid__: jid to run under
                 - __tag__: tag to run under
         '''
-        jid = low.get('__jid__', salt.utils.jid.gen_jid())
-        tag = low.get('__tag__', tagify(jid, prefix=self.tag_prefix))
+        # fire the mminion loading (if not already done) here
+        # this is not to clutter the output with the module loading
+        # if we have a high debug level.
+        self.mminion  # pylint: disable=W0104
+        jid = low.get('__jid__', salt.utils.jid.gen_jid(self.opts))
+        tag = low.get('__tag__', salt.utils.event.tagify(jid, prefix=self.tag_prefix))
 
         data = {'fun': '{0}.{1}'.format(self.client, fun),
                 'jid': jid,
                 'user': low.get('__user__', 'UNKNOWN'),
-                }
+               }
 
         event = salt.utils.event.get_event(
                 'master',
@@ -244,13 +314,21 @@ class SyncClientMixin(object):
                 opts=self.opts,
                 listen=False)
 
+        if print_event:
+            print_func = self.print_async_event \
+                if hasattr(self, 'print_async_event') \
+                else None
+        else:
+            # Suppress printing of return event (this keeps us from printing
+            # runner/wheel output during orchestration).
+            print_func = None
+
         namespaced_event = salt.utils.event.NamespacedEvent(
             event,
             tag,
-            print_func=self.print_async_event
-                       if hasattr(self, 'print_async_event')
-                       else None
+            print_func=print_func
         )
+
         # TODO: document these, and test that they exist
         # TODO: Other things to inject??
         func_globals = {'__jid__': jid,
@@ -261,101 +339,121 @@ class SyncClientMixin(object):
                         '__jid_event__': weakref.proxy(namespaced_event),
                         }
 
-        func_globals['__jid_event__'].fire_event(data, 'new')
-
         try:
-            verify_fun(self.functions, fun)
+            self_functions = pycopy.copy(self.functions)
+            salt.utils.lazy.verify_fun(self_functions, fun)
 
-            # Inject some useful globals to *all* the funciton's global
+            # Inject some useful globals to *all* the function's global
             # namespace only once per module-- not per func
             completed_funcs = []
-            for mod_name in self.functions:
+
+            for mod_name in six.iterkeys(self_functions):
+                if '.' not in mod_name:
+                    continue
                 mod, _ = mod_name.split('.', 1)
                 if mod in completed_funcs:
                     continue
                 completed_funcs.append(mod)
-                for global_key, value in func_globals.iteritems():
+                for global_key, value in six.iteritems(func_globals):
                     self.functions[mod_name].__globals__[global_key] = value
 
-            # There are some descrepencies of what a "low" structure is in the
+            # There are some discrepancies of what a "low" structure is in the
             # publisher world it is a dict including stuff such as jid, fun,
             # arg (a list of args, with kwargs packed in). Historically this
             # particular one has had no "arg" and just has had all the kwargs
             # packed into the top level object. The plan is to move away from
             # that since the caller knows what is an arg vs a kwarg, but while
             # we make the transition we will load "kwargs" using format_call if
-            # there are no kwargs in the low object passed in
-            f_call = None
-            if 'args' not in low:
-                f_call = salt.utils.format_call(
+            # there are no kwargs in the low object passed in.
+
+            if 'arg' in low and 'kwarg' in low:
+                args = low['arg']
+                kwargs = low['kwarg']
+            else:
+                f_call = salt.utils.args.format_call(
                     self.functions[fun],
                     low,
                     expected_extra_kws=CLIENT_INTERNAL_KEYWORDS
                 )
                 args = f_call.get('args', ())
-            else:
-                args = low['args']
-            if 'kwargs' not in low:
-                if f_call is None:
-                    f_call = salt.utils.format_call(
-                        self.functions[fun],
-                        low,
-                        expected_extra_kws=CLIENT_INTERNAL_KEYWORDS
-                    )
                 kwargs = f_call.get('kwargs', {})
 
-                # throw a warning for the badly formed low data if we found
-                # kwargs using the old mechanism
-                if kwargs:
-                    salt.utils.warn_until(
-                        'Boron',
-                        'kwargs must be passed inside the low under "kwargs"'
-                    )
-            else:
-                kwargs = low['kwargs']
+            # Update the event data with loaded args and kwargs
+            data['fun_args'] = list(args) + ([kwargs] if kwargs else [])
+            func_globals['__jid_event__'].fire_event(data, 'new')
 
-            data['return'] = self.functions[fun](*args, **kwargs)
-            data['success'] = True
-        except (Exception, SystemExit) as exc:
-            data['return'] = 'Exception occurred in {0} {1}: {2}'.format(
-                            self.client,
-                            fun,
-                            traceback.format_exc(),
-                            )
+            # Initialize a context for executing the method.
+            with tornado.stack_context.StackContext(self.functions.context_dict.clone):
+                data['return'] = self.functions[fun](*args, **kwargs)
+                try:
+                    data['success'] = self.context.get('retcode', 0) == 0
+                except AttributeError:
+                    # Assume a True result if no context attribute
+                    data['success'] = True
+                if isinstance(data['return'], dict) and 'data' in data['return']:
+                    # some functions can return boolean values
+                    data['success'] = salt.utils.state.check_result(data['return']['data'])
+        except (Exception, SystemExit) as ex:
+            if isinstance(ex, salt.exceptions.NotImplemented):
+                data['return'] = six.text_type(ex)
+            else:
+                data['return'] = 'Exception occurred in {0} {1}: {2}'.format(
+                    self.client,
+                    fun,
+                    traceback.format_exc(),
+                    )
             data['success'] = False
 
+        if self.store_job:
+            try:
+                salt.utils.job.store_job(
+                    self.opts,
+                    {
+                        'id': self.opts['id'],
+                        'tgt': self.opts['id'],
+                        'jid': data['jid'],
+                        'return': data,
+                    },
+                    event=None,
+                    mminion=self.mminion,
+                    )
+            except salt.exceptions.SaltCacheError:
+                log.error('Could not store job cache info. '
+                          'Job details for this run may be unavailable.')
+
+        # Outputters _can_ mutate data so write to the job cache first!
         namespaced_event.fire_event(data, 'ret')
-        salt.utils.job.store_job(
-            self.opts,
-            {'id': self.opts['id'],
-             'tgt': self.opts['id'],
-             'jid': data['jid'],
-             'return': data,
-             },
-            event=None,
-            mminion=self.mminion,
-            )
+
         # if we fired an event, make sure to delete the event object.
         # This will ensure that we call destroy, which will do the 0MQ linger
-        log.info('Runner completed: {0}'.format(data['jid']))
+        log.info('Runner completed: %s', data['jid'])
         del event
         del namespaced_event
-        return data['return']
+        return data if full_return else data['return']
 
     def get_docs(self, arg=None):
         '''
         Return a dictionary of functions and the inline documentation for each
         '''
         if arg:
-            target_mod = arg + '.' if not arg.endswith('.') else arg
-            docs = [(fun, self.functions[fun].__doc__)
-                    for fun in sorted(self.functions)
-                    if fun == arg or fun.startswith(target_mod)]
+            if '*' in arg:
+                target_mod = arg
+                _use_fnmatch = True
+            else:
+                target_mod = arg + '.' if not arg.endswith('.') else arg
+                _use_fnmatch = False
+            if _use_fnmatch:
+                docs = [(fun, self.functions[fun].__doc__)
+                        for fun in fnmatch.filter(self.functions, target_mod)]
+            else:
+                docs = [(fun, self.functions[fun].__doc__)
+                        for fun in sorted(self.functions)
+                        if fun == arg or fun.startswith(target_mod)]
         else:
             docs = [(fun, self.functions[fun].__doc__)
                     for fun in sorted(self.functions)]
         docs = dict(docs)
-        return _strip_rst(docs)
+        return salt.utils.doc.strip_rst(docs)
 
 
 class AsyncClientMixin(object):
@@ -370,15 +468,21 @@ class AsyncClientMixin(object):
         Run this method in a multiprocess target to execute the function in a
         multiprocess and fire the return data on the event bus
         '''
-        if daemonize:
-            salt.utils.daemonize()
+        if daemonize and not salt.utils.platform.is_windows():
+            # Shutdown the multiprocessing before daemonizing
+            salt.log.setup.shutdown_multiprocessing_logging()
+
+            salt.utils.process.daemonize()
+
+            # Reconfigure multiprocessing logging after daemonizing
+            salt.log.setup.setup_multiprocessing_logging()
 
         # pack a few things into low
         low['__jid__'] = jid
         low['__user__'] = user
         low['__tag__'] = tag
 
-        return self.low(fun, low)
+        return self.low(fun, low, full_return=False)
 
     def cmd_async(self, low):
         '''
@@ -402,21 +506,24 @@ class AsyncClientMixin(object):
 
     def _gen_async_pub(self, jid=None):
         if jid is None:
-            jid = salt.utils.jid.gen_jid()
-        tag = tagify(jid, prefix=self.tag_prefix)
+            jid = salt.utils.jid.gen_jid(self.opts)
+        tag = salt.utils.event.tagify(jid, prefix=self.tag_prefix)
         return {'tag': tag, 'jid': jid}
 
-    def async(self, fun, low, user='UNKNOWN'):
+    def async(self, fun, low, user='UNKNOWN', pub=None):
         '''
         Execute the function in a multiprocess and return the event tag to use
         to watch for the return
         '''
-        async_pub = self._gen_async_pub()
+        async_pub = pub if pub is not None else self._gen_async_pub()
 
-        proc = multiprocessing.Process(
+        proc = salt.utils.process.SignalHandlingMultiprocessingProcess(
                 target=self._proc_function,
                 args=(fun, low, user, async_pub['tag'], async_pub['jid']))
-        proc.start()
+        with salt.utils.process.default_signals(signal.SIGINT, signal.SIGTERM):
+            # Reset current signals before starting the process in
+            # order not to inherit the current signal handlers
+            proc.start()
         proc.join()  # MUST join, otherwise we leave zombies all over
         return async_pub
 
@@ -435,10 +542,14 @@ class AsyncClientMixin(object):
         if suffix in ('new',):
             return
 
-        outputter = self.opts.get('output', event.get('outputter', None))
+        try:
+            outputter = self.opts.get('output', event.get('outputter', None) or event.get('return').get('outputter'))
+        except AttributeError:
+            outputter = None
+
         # if this is a ret, we have our own set of rules
         if suffix == 'ret':
-            # Check if ouputter was passed in the return data. If this is the case,
+            # Check if outputter was passed in the return data. If this is the case,
             # then the return data will be a dict two keys: 'data' and 'outputter'
             if isinstance(event.get('return'), dict) \
                     and set(event['return']) == set(('data', 'outputter')):
