@@ -12,7 +12,7 @@ Module to import docker-compose via saltstack
 
 Introduction
 ------------
-This module allows to deal with docker-compose file in a directory.
+This module allows one to deal with docker-compose file in a directory.
 
 This is  a first version only, the following commands are missing at the moment
 but will be built later on if the community is interested in this module:
@@ -93,20 +93,24 @@ Functions
 - Manage containers image:
     - :py:func:`dockercompose.pull <salt.modules.dockercompose.pull>`
     - :py:func:`dockercompose.build <salt.modules.dockercompose.build>`
-- Gather informations about containers:
+- Gather information about containers:
     - :py:func:`dockercompose.ps <salt.modules.dockercompose.ps>`
 
 Detailed Function Documentation
 -------------------------------
 '''
 
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 import inspect
 import logging
 import os
 import re
-import salt.utils
+
+import salt.utils.files
+import salt.utils.stringutils
+
+from salt.ext import six
 
 from operator import attrgetter
 try:
@@ -117,6 +121,12 @@ try:
 except ImportError:
     HAS_DOCKERCOMPOSE = False
 
+try:
+    from compose.project import OneOffFilter
+    USE_FILTERCLASS = True
+except ImportError:
+    USE_FILTERCLASS = False
+
 MIN_DOCKERCOMPOSE = (1, 5, 0)
 VERSION_RE = r'([\d.]+)'
 
@@ -124,18 +134,16 @@ log = logging.getLogger(__name__)
 debug = False
 
 __virtualname__ = 'dockercompose'
-dc_filename = 'docker-compose.yml'
+DEFAULT_DC_FILENAMES = ('docker-compose.yml', 'docker-compose.yaml')
 
 
 def __virtual__():
     if HAS_DOCKERCOMPOSE:
-        match = re.match(VERSION_RE, str(compose.__version__))
+        match = re.match(VERSION_RE, six.text_type(compose.__version__))
         if match:
             version = tuple([int(x) for x in match.group(1).split('.')])
-            if MIN_DOCKERCOMPOSE >= version:
+            if version >= MIN_DOCKERCOMPOSE:
                 return __virtualname__
-        else:
-            log.critical('Minimum version of docker-compose>=1.5.0')
     return (False, 'The dockercompose execution module not loaded: '
             'compose python library not available.')
 
@@ -164,27 +172,45 @@ def __standardize_result(status, message, data=None, debug_msg=None):
     return result
 
 
-def __read_docker_compose(path):
+def __get_docker_file_path(path):
     '''
-    Read the docker-compose.yml file if it exists in the directory
+    Determines the filepath to use
 
     :param path:
     :return:
     '''
-    if os.path.isfile(os.path.join(path, dc_filename)) is False:
+    if os.path.isfile(path):
+        return path
+    for dc_filename in DEFAULT_DC_FILENAMES:
+        file_path = os.path.join(path, dc_filename)
+        if os.path.isfile(file_path):
+            return file_path
+    # implicitly return None
+
+
+def __read_docker_compose_file(file_path):
+    '''
+    Read the compose file if it exists in the directory
+
+    :param file_path:
+    :return:
+    '''
+    if not os.path.isfile(file_path):
         return __standardize_result(False,
-                                    'Path does not exist or docker-compose.yml is not present',
+                                    'Path {} is not present'.format(file_path),
                                     None, None)
-    f = salt.utils.fopen(os.path.join(path, dc_filename), 'r')
-    result = {'docker-compose.yml': ''}
-    if f:
-        for line in f:
-            result['docker-compose.yml'] += line
-        f.close()
-    else:
-        return __standardize_result(False, 'Could not read docker-compose.yml file.',
+    try:
+        with salt.utils.files.fopen(file_path, 'r') as fl:
+            file_name = os.path.basename(file_path)
+            result = {file_name: ''}
+            for line in fl:
+                result[file_name] += salt.utils.stringutils.to_unicode(line)
+    except EnvironmentError:
+        return __standardize_result(False,
+                                    'Could not read {0}'.format(file_path),
                                     None, None)
-    return __standardize_result(True, 'Reading content of docker-compose.yml file',
+    return __standardize_result(True,
+                                'Reading content of {0}'.format(file_path),
                                 result, None)
 
 
@@ -200,33 +226,54 @@ def __write_docker_compose(path, docker_compose):
 
     :return:
     '''
-
-    if os.path.isdir(path) is False:
-        os.mkdir(path)
-    f = salt.utils.fopen(os.path.join(path, dc_filename), 'w')
-    if f:
-        f.write(docker_compose)
-        f.close()
+    if path.lower().endswith(('.yml', '.yaml')):
+        file_path = path
+        dir_name = os.path.dirname(path)
     else:
+        dir_name = path
+        file_path = os.path.join(dir_name, DEFAULT_DC_FILENAMES[0])
+    if os.path.isdir(dir_name) is False:
+        os.mkdir(dir_name)
+    try:
+        with salt.utils.files.fopen(file_path, 'w') as fl:
+            fl.write(salt.utils.stringutils.to_str(docker_compose))
+    except EnvironmentError:
         return __standardize_result(False,
-                                    'Could not write docker-compose file in {0}'.format(path),
+                                    'Could not write {0}'.format(file_path),
                                     None, None)
-    project = __load_project(path)
+    project = __load_project_from_file_path(file_path)
     if isinstance(project, dict):
-        os.remove(os.path.join(path, dc_filename))
+        os.remove(file_path)
         return project
-    return path
+    return file_path
 
 
 def __load_project(path):
     '''
     Load a docker-compose project from path
 
+    :param path:
+    :return:
+    '''
+    file_path = __get_docker_file_path(path)
+    if file_path is None:
+        msg = 'Could not find docker-compose file at {0}'.format(path)
+        return __standardize_result(False,
+                                    msg,
+                                    None, None)
+    return __load_project_from_file_path(file_path)
+
+
+def __load_project_from_file_path(file_path):
+    '''
+    Load a docker-compose project from file path
+
     :param path:
     :return:
     '''
     try:
-        project = get_project(path)
+        project = get_project(project_dir=os.path.dirname(file_path),
+                              config_path=[os.path.basename(file_path)])
     except Exception as inst:
         return __handle_except(inst)
     return project
@@ -234,7 +281,7 @@ def __load_project(path):
 
 def __handle_except(inst):
     '''
-    Handle exception and return a standart result
+    Handle exception and return a standard result
 
     :param inst:
     :return:
@@ -282,8 +329,12 @@ def get(path):
 
         salt myminion dockercompose.get /path/where/docker-compose/stored
     '''
-
-    salt_result = __read_docker_compose(path)
+    file_path = __get_docker_file_path(path)
+    if file_path is None:
+        return __standardize_result(False,
+                                    'Path {} is not present'.format(path),
+                                    None, None)
+    salt_result = __read_docker_compose_file(file_path)
     if not salt_result['status']:
         return salt_result
     project = __load_project(path)
@@ -318,7 +369,10 @@ def create(path, docker_compose):
         return __standardize_result(False,
                                     'Creating a docker-compose project failed, you must send a valid docker-compose file',
                                     None, None)
-    return __standardize_result(True, 'Successfully created the docker-compose file', {'compose.base_dir': path}, None)
+    return __standardize_result(True,
+                                'Successfully created the docker-compose file',
+                                {'compose.base_dir': path},
+                                None)
 
 
 def pull(path, service_names=None):
@@ -357,7 +411,7 @@ def build(path, service_names=None):
     python list, if omitted build images for all containers. Please note
     that at the moment the module does not allow you to upload your Dockerfile,
     nor any other file you could need with your docker-compose.yml, you will
-    have to make sure the files you need are actually in the directory sepcified
+    have to make sure the files you need are actually in the directory specified
     in the `build` keyword
 
     path
@@ -656,10 +710,16 @@ def ps(path):
     if isinstance(project, dict):
         return project
     else:
-        containers = sorted(
-            project.containers(None, stopped=True) +
-            project.containers(None, one_off=True),
-            key=attrgetter('name'))
+        if USE_FILTERCLASS:
+            containers = sorted(
+                project.containers(None, stopped=True) +
+                project.containers(None, OneOffFilter.only),
+                key=attrgetter('name'))
+        else:
+            containers = sorted(
+                project.containers(None, stopped=True) +
+                project.containers(None, one_off=True),
+                key=attrgetter('name'))
         for container in containers:
             command = container.human_readable_command
             if len(command) > 30:
@@ -676,7 +736,7 @@ def ps(path):
 
 def up(path, service_names=None):
     '''
-    Create and start containers defined in the the docker-compose.yml file
+    Create and start containers defined in the docker-compose.yml file
     located in path, service_names is a python list, if omitted create and
     start all containers
 
