@@ -2,21 +2,23 @@
 '''
 Manage and query NPM packages.
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 try:
     from shlex import quote as _cmd_quote  # pylint: disable=E0611
 except ImportError:
     from pipes import quote as _cmd_quote
 
 # Import python libs
-import json
 import logging
-import distutils.version  # pylint: disable=import-error,no-name-in-module
 
 # Import salt libs
-import salt.utils
+import salt.utils.json
+import salt.utils.path
+import salt.utils.user
 import salt.modules.cmdmod
 from salt.exceptions import CommandExecutionError
+from salt.utils.versions import LooseVersion as _LooseVersion
+from salt.ext import six
 
 
 log = logging.getLogger(__name__)
@@ -32,14 +34,14 @@ def __virtual__():
     Only work when npm is installed.
     '''
     try:
-        if salt.utils.which('npm') is not None:
+        if salt.utils.path.which('npm') is not None:
             _check_valid_version()
             return True
         else:
             return (False, 'npm execution module could not be loaded '
                            'because the npm binary could not be located')
     except CommandExecutionError as exc:
-        return (False, str(exc))
+        return (False, six.text_type(exc))
 
 
 def _check_valid_version():
@@ -48,9 +50,9 @@ def _check_valid_version():
     npm must be at least version 1.2.
     '''
     # pylint: disable=no-member
-    npm_version = distutils.version.LooseVersion(
+    npm_version = _LooseVersion(
         salt.modules.cmdmod.run('npm --version', output_loglevel='quiet'))
-    valid_version = distutils.version.LooseVersion('1.2')
+    valid_version = _LooseVersion('1.2')
     # pylint: enable=no-member
     if npm_version < valid_version:
         raise CommandExecutionError(
@@ -155,9 +157,9 @@ def install(pkg=None,
     env = env or {}
 
     if runas:
-        uid = salt.utils.get_uid(runas)
+        uid = salt.utils.user.get_uid(runas)
         if uid:
-            env.update({'SUDO_UID': b'{0}'.format(uid), 'SUDO_USER': b''})
+            env.update({'SUDO_UID': uid, 'SUDO_USER': ''})
 
     cmd = ' '.join(cmd)
     result = __salt__['cmd.run_all'](cmd, python_shell=True, cwd=dir, runas=runas, env=env)
@@ -168,7 +170,7 @@ def install(pkg=None,
     # npm >1.2.21 is putting the output to stderr even though retcode is 0
     npm_output = result['stdout'] or result['stderr']
     try:
-        return json.loads(npm_output)
+        return salt.utils.json.loads(npm_output)
     except ValueError:
         pass
 
@@ -188,10 +190,10 @@ def _extract_json(npm_output):
     # macOS with fsevents includes the following line in the return
     # when a new module is installed which is invalid JSON:
     #     [fsevents] Success: "..."
-    while lines and lines[0].startswith('[fsevents]'):
+    while lines and (lines[0].startswith('[fsevents]') or lines[0].startswith('Pass ')):
         lines = lines[1:]
     try:
-        return json.loads(''.join(lines))
+        return salt.utils.json.loads(''.join(lines))
     except ValueError:
         pass
     return None
@@ -234,9 +236,9 @@ def uninstall(pkg, dir=None, runas=None, env=None):
     env = env or {}
 
     if runas:
-        uid = salt.utils.get_uid(runas)
+        uid = salt.utils.user.get_uid(runas)
         if uid:
-            env.update({'SUDO_UID': b'{0}'.format(uid), 'SUDO_USER': b''})
+            env.update({'SUDO_UID': uid, 'SUDO_USER': ''})
 
     cmd = ['npm', 'uninstall', '"{0}"'.format(pkg)]
     if not dir:
@@ -252,7 +254,7 @@ def uninstall(pkg, dir=None, runas=None, env=None):
     return True
 
 
-def list_(pkg=None, dir=None, runas=None, env=None):
+def list_(pkg=None, dir=None, runas=None, env=None, depth=None):
     '''
     List installed NPM packages.
 
@@ -278,6 +280,11 @@ def list_(pkg=None, dir=None, runas=None, env=None):
 
         .. versionadded:: 2014.7.0
 
+    depth
+        Limit the depth of the packages listed
+
+        .. versionadded:: 2016.11.6, 2017.7.0
+
     CLI Example:
 
     .. code-block:: bash
@@ -288,15 +295,20 @@ def list_(pkg=None, dir=None, runas=None, env=None):
     env = env or {}
 
     if runas:
-        uid = salt.utils.get_uid(runas)
+        uid = salt.utils.user.get_uid(runas)
         if uid:
-            env.update({'SUDO_UID': b'{0}'.format(uid), 'SUDO_USER': b''})
+            env.update({'SUDO_UID': uid, 'SUDO_USER': ''})
 
     cmd = ['npm', 'list', '--json', '--silent']
 
     if not dir:
         cmd.append('--global')
 
+    if depth is not None:
+        if not isinstance(depth, (int, float)):
+            raise salt.exceptions.SaltInvocationError('Error: depth {0} must be a number'.format(depth))
+        cmd.append('--depth={0}'.format(int(depth)))
+
     if pkg:
         # Protect against injection
         pkg = _cmd_quote(pkg)
@@ -311,10 +323,10 @@ def list_(pkg=None, dir=None, runas=None, env=None):
     if result['retcode'] != 0 and result['stderr']:
         raise CommandExecutionError(result['stderr'])
 
-    return json.loads(result['stdout']).get('dependencies', {})
+    return salt.utils.json.loads(result['stdout']).get('dependencies', {})
 
 
-def cache_clean(path=None, runas=None, env=None):
+def cache_clean(path=None, runas=None, env=None, force=False):
     '''
     Clean cached NPM packages.
 
@@ -331,23 +343,30 @@ def cache_clean(path=None, runas=None, env=None):
         format as the :py:func:`cmd.run <salt.modules.cmdmod.run>` execution
         function.
 
+    force
+        Force cleaning of cache.  Required for npm@5 and greater
+
+        .. versionadded:: 2016.11.6
+
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' npm.cache_clean
+        salt '*' npm.cache_clean force=True
 
     '''
     env = env or {}
 
     if runas:
-        uid = salt.utils.get_uid(runas)
+        uid = salt.utils.user.get_uid(runas)
         if uid:
-            env.update({'SUDO_UID': b'{0}'.format(uid), 'SUDO_USER': b''})
+            env.update({'SUDO_UID': uid, 'SUDO_USER': ''})
 
     cmd = ['npm', 'cache', 'clean']
     if path:
         cmd.append(path)
+    if force is True:
+        cmd.append('--force')
 
     cmd = ' '.join(cmd)
     result = __salt__['cmd.run_all'](
@@ -386,9 +405,9 @@ def cache_list(path=None, runas=None, env=None):
     env = env or {}
 
     if runas:
-        uid = salt.utils.get_uid(runas)
+        uid = salt.utils.user.get_uid(runas)
         if uid:
-            env.update({'SUDO_UID': b'{0}'.format(uid), 'SUDO_USER': b''})
+            env.update({'SUDO_UID': uid, 'SUDO_USER': ''})
 
     cmd = ['npm', 'cache', 'ls']
     if path:
@@ -426,9 +445,9 @@ def cache_path(runas=None, env=None):
     env = env or {}
 
     if runas:
-        uid = salt.utils.get_uid(runas)
+        uid = salt.utils.user.get_uid(runas)
         if uid:
-            env.update({'SUDO_UID': b'{0}'.format(uid), 'SUDO_USER': b''})
+            env.update({'SUDO_UID': uid, 'SUDO_USER': ''})
 
     cmd = 'npm config get cache'
 
