@@ -1,14 +1,27 @@
 # -*- coding: utf-8 -*-
 '''
 The service module for OpenBSD
+
+.. important::
+    If you feel that Salt should be using this module to manage services on a
+    minion, and it is using a different module (or gives an error similar to
+    *'service.start' is not available*), see :ref:`here
+    <module-provider-override>`.
 '''
 
 # Import python libs
+from __future__ import absolute_import, print_function, unicode_literals
 import os
+import fnmatch
 import logging
 
+# Import 3rd-party libs
+from salt.ext import six
+from salt.ext.six.moves import map  # pylint: disable=import-error,redefined-builtin
+
 # Import Salt libs
-import salt.utils
+import salt.utils.data
+import salt.utils.files
 
 log = logging.getLogger(__name__)
 
@@ -27,14 +40,15 @@ def __virtual__():
     Only work on OpenBSD
     '''
     if __grains__['os'] == 'OpenBSD' and os.path.exists('/etc/rc.d/rc.subr'):
-        krel = map(int, __grains__['kernelrelease'].split('.'))
+        krel = list(list(map(int, __grains__['kernelrelease'].split('.'))))
         # The -f flag, used to force a script to run even if disabled,
         # was added after the 5.0 release.
-        # the rcctl(8) command is the prefered way to manage services.
+        # the rcctl(8) command is the preferred way to manage services.
         if krel[0] > 5 or (krel[0] == 5 and krel[1] > 0):
             if not os.path.exists('/usr/sbin/rcctl'):
                 return __virtualname__
-    return False
+    return (False, 'The openbsdservice execution module cannot be loaded: '
+            'only available on OpenBSD systems.')
 
 
 def start(name):
@@ -81,19 +95,42 @@ def restart(name):
 
 def status(name, sig=None):
     '''
-    Return the status for a service, returns a bool whether the service is
-    running.
+    Return the status for a service.
+    If the name contains globbing, a dict mapping service name to True/False
+    values is returned.
+
+    .. versionchanged:: 2018.3.0
+        The service name can now be a glob (e.g. ``salt*``)
+
+    Args:
+        name (str): The name of the service to check
+        sig (str): Signature to use to find the service via ps
+
+    Returns:
+        bool: True if running, False otherwise
+        dict: Maps service name to True if running, False otherwise
 
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' service.status <service name>
+        salt '*' service.status <service name> [service signature]
     '''
     if sig:
         return bool(__salt__['status.pid'](sig))
-    cmd = '/etc/rc.d/{0} -f check'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+
+    contains_globbing = bool(re.search(r'\*|\?|\[.+\]', name))
+    if contains_globbing:
+        services = fnmatch.filter(get_all(), name)
+    else:
+        services = [name]
+    results = {}
+    for service in services:
+        cmd = '/etc/rc.d/{0} -f check'.format(service)
+        results[service] = not __salt__['cmd.retcode'](cmd, ignore_retcode=True)
+    if contains_globbing:
+        return results
+    return results[name]
 
 
 def reload_(name):
@@ -132,8 +169,8 @@ def _get_rc():
     try:
         # now read the system startup script /etc/rc
         # to know what are the system enabled daemons
-        with salt.utils.fopen('/etc/rc', 'r') as handle:
-            lines = handle.readlines()
+        with salt.utils.files.fopen('/etc/rc', 'r') as handle:
+            lines = salt.utils.data.decode(handle.readlines())
     except IOError:
         log.error('Unable to read /etc/rc')
     else:
@@ -243,13 +280,13 @@ def get_enabled():
         salt '*' service.get_enabled
     '''
     services = []
-    for daemon, is_enabled in _get_rc().items():
+    for daemon, is_enabled in six.iteritems(_get_rc()):
         if is_enabled:
             services.append(daemon)
     return sorted(set(get_all()) & set(services))
 
 
-def enabled(name):
+def enabled(name, **kwargs):
     '''
     .. versionadded:: 2014.7.0
 
@@ -277,7 +314,7 @@ def get_disabled():
         salt '*' service.get_disabled
     '''
     services = []
-    for daemon, is_enabled in _get_rc().items():
+    for daemon, is_enabled in six.iteritems(_get_rc()):
         if not is_enabled:
             services.append(daemon)
     return sorted(set(get_all()) & set(services))
