@@ -2,7 +2,7 @@
 '''
 Extract the pillar data for this minion
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import collections
@@ -12,13 +12,16 @@ import copy
 import os
 import copy
 import logging
-import yaml
-import salt.ext.six as six
+from salt.ext import six
 
 # Import salt libs
 import salt.pillar
-import salt.utils
 import salt.utils.crypt
+import salt.utils.data
+import salt.utils.dictupdate
+import salt.utils.functools
+import salt.utils.odict
+import salt.utils.yaml
 from salt.defaults import DEFAULT_TARGET_DELIM
 from salt.exceptions import CommandExecutionError
 
@@ -39,8 +42,8 @@ def get(key,
 
     Attempt to retrieve the named value from pillar, if the named value is not
     available return the passed default. The default return is an empty string
-    except __opts__['pillar_raise_on_missing'] is set to True, in which case a
-    KeyError will be raised.
+    except ``__opts__['pillar_raise_on_missing']`` is set to True, in which
+    case a ``KeyError`` exception will be raised.
 
     If the merge parameter is set to ``True``, the default will be recursively
     merged into the returned pillar data.
@@ -50,11 +53,18 @@ def get(key,
 
         {'pkg': {'apache': 'httpd'}}
 
-    To retrieve the value associated with the apache key in the pkg dict this
-    key can be passed::
+    To retrieve the value associated with the ``apache`` key in the ``pkg``
+    dict this key can be passed as::
 
         pkg:apache
 
+    key
+        The pillar key to get value from
+
+    default
+        If specified, return this value in case when named pillar value does
+        not exist.
+
     merge : ``False``
         If ``True``, the retrieved values will be merged into the passed
         default. When the default and the retrieved value are both
@@ -125,7 +135,7 @@ def get(key,
 
     if merge:
         if isinstance(default, dict):
-            ret = salt.utils.traverse_dict_and_list(
+            ret = salt.utils.data.traverse_dict_and_list(
                 pillar_dict,
                 key,
                 {},
@@ -143,7 +153,7 @@ def get(key,
                     'skipped.', default, ret, type(ret).__name__
                 )
         elif isinstance(default, list):
-            ret = salt.utils.traverse_dict_and_list(
+            ret = salt.utils.data.traverse_dict_and_list(
                 pillar_dict,
                 key,
                 [],
@@ -165,10 +175,11 @@ def get(key,
                 default, type(default).__name__
             )
 
-    ret = salt.utils.traverse_dict_and_list(pillar_dict,
-                                            key,
-                                            default,
-                                            delimiter)
+    ret = salt.utils.data.traverse_dict_and_list(
+        pillar_dict,
+        key,
+        default,
+        delimiter)
     if ret is KeyError:
         raise KeyError('Pillar key not found: {0}'.format(key))
 
@@ -233,7 +244,7 @@ def items(*args, **kwargs):
     pillarenv = kwargs.get('pillarenv')
     if pillarenv is None:
         if __opts__.get('pillarenv_from_saltenv', False):
-            pillarenv = kwargs.get('saltenv') or __opts__['environment']
+            pillarenv = kwargs.get('saltenv') or __opts__['saltenv']
         else:
             pillarenv = __opts__['pillarenv']
 
@@ -263,7 +274,7 @@ def items(*args, **kwargs):
     return pillar.compile_pillar()
 
 # Allow pillar.data to also be used to return pillar data
-data = salt.utils.alias_function(items, 'data')
+data = salt.utils.functools.alias_function(items, 'data')
 
 
 def _obfuscate_inner(var):
@@ -355,6 +366,28 @@ def item(*args, **kwargs):
 
         .. versionadded:: 2015.8.0
 
+    pillarenv
+        If specified, this function will query the master to generate fresh
+        pillar data on the fly, specifically from the requested pillar
+        environment. Note that this can produce different pillar data than
+        executing this function without an environment, as its normal behavior
+        is just to return a value from minion's pillar data in memory (which
+        can be sourced from more than one pillar environment).
+
+        Using this argument will not affect the pillar data in memory. It will
+        however be slightly slower and use more resources on the master due to
+        the need for the master to generate and send the minion fresh pillar
+        data. This tradeoff in performance however allows for the use case
+        where pillar data is desired only from a single environment.
+
+        .. versionadded:: 2017.7.6,2018.3.1
+
+    saltenv
+        Included only for compatibility with
+        :conf_minion:`pillarenv_from_saltenv`, and is otherwise ignored.
+
+        .. versionadded:: 2017.7.6,2018.3.1
+
     CLI Examples:
 
     .. code-block:: bash
@@ -366,13 +399,20 @@ def item(*args, **kwargs):
     ret = {}
     default = kwargs.get('default', '')
     delimiter = kwargs.get('delimiter', DEFAULT_TARGET_DELIM)
+    pillarenv = kwargs.get('pillarenv', None)
+    saltenv = kwargs.get('saltenv', None)
+
+    pillar_dict = __pillar__ \
+        if all(x is None for x in (saltenv, pillarenv)) \
+        else items(saltenv=saltenv, pillarenv=pillarenv)
 
     try:
         for arg in args:
-            ret[arg] = salt.utils.traverse_dict_and_list(__pillar__,
-                                                         arg,
-                                                         default,
-                                                         delimiter)
+            ret[arg] = salt.utils.data.traverse_dict_and_list(
+                pillar_dict,
+                arg,
+                default,
+                delimiter)
     except KeyError:
         pass
 
@@ -424,8 +464,8 @@ def ext(external, pillar=None):
 
         .. code-block:: python
 
-            >>> import yaml
-            >>> ext_pillar = yaml.safe_load("""
+            >>> import salt.utils.yaml
+            >>> ext_pillar = salt.utils.yaml.safe_load("""
             ... ext_pillar:
             ...   - git:
             ...     - issue38440 https://github.com/terminalmage/git_pillar:
@@ -458,12 +498,12 @@ def ext(external, pillar=None):
         salt '*' pillar.ext "{'git': [{'mybranch https://github.com/myuser/myrepo': [{'env': 'base'}]}]}"
     '''
     if isinstance(external, six.string_types):
-        external = yaml.safe_load(external)
+        external = salt.utils.yaml.safe_load(external)
     pillar_obj = salt.pillar.get_pillar(
         __opts__,
         __grains__,
         __opts__['id'],
-        __opts__['environment'],
+        __opts__['saltenv'],
         ext=external,
         pillar_override=pillar)
 
@@ -490,7 +530,7 @@ def keys(key, delimiter=DEFAULT_TARGET_DELIM):
 
         salt '*' pillar.keys web:sites
     '''
-    ret = salt.utils.traverse_dict_and_list(
+    ret = salt.utils.data.traverse_dict_and_list(
         __pillar__, key, KeyError, delimiter)
 
     if ret is KeyError:
@@ -606,9 +646,9 @@ def filter_by(lookup_dict,
 
         salt '*' pillar.filter_by '{web: Serve it up, db: I query, default: x_x}' role
     '''
-    return salt.utils.filter_by(lookup_dict=lookup_dict,
-                                lookup=pillar,
-                                traverse=__pillar__,
-                                merge=merge,
-                                default=default,
-                                base=base)
+    return salt.utils.data.filter_by(lookup_dict=lookup_dict,
+                                     lookup=pillar,
+                                     traverse=__pillar__,
+                                     merge=merge,
+                                     default=default,
+                                     base=base)
