@@ -45,7 +45,7 @@ Service's alias:
     XBPS package management uses a service's alias to provides service
     alternative(s), such as chrony and openntpd both aliased to ntpd.
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import python libs
 import os
@@ -57,7 +57,8 @@ log = logging.getLogger(__name__)
 
 # Import salt libs
 from salt.exceptions import CommandExecutionError
-import salt.utils
+import salt.utils.files
+import salt.utils.path
 
 # Function alias to not shadow built-ins.
 __func_alias__ = {
@@ -80,7 +81,8 @@ for service_dir in VALID_SERVICE_DIRS:
 AVAIL_SVR_DIRS = []
 
 # Define the module's virtual name
-__virtualname__ = 'service'
+__virtualname__ = 'runit'
+__virtual_aliases__ = ('runit',)
 
 
 def __virtual__():
@@ -91,8 +93,12 @@ def __virtual__():
     if __grains__.get('init') == 'runit':
         if __grains__['os'] == 'Void':
             add_svc_avail_path('/etc/sv')
+        global __virtualname__
+        __virtualname__ = 'service'
         return __virtualname__
-    return False
+    if salt.utils.path.which('sv'):
+        return __virtualname__
+    return (False, 'Runit not available.  Please install sv')
 
 
 def _service_path(name):
@@ -345,7 +351,7 @@ def _get_svc_path(name='*', status=None):
     for el in glob.glob(os.path.join(SERVICE_DIR, name)):
         if _is_svc(el):
             ena.add(os.readlink(el))
-            log.trace('found enabled service path: {0}'.format(el))
+            log.trace('found enabled service path: %s', el)
 
     if status == 'ENABLED':
         return sorted(ena)
@@ -356,7 +362,7 @@ def _get_svc_path(name='*', status=None):
         for el in glob.glob(os.path.join(d, name)):
             if _is_svc(el):
                 ava.add(el)
-                log.trace('found available service path: {0}'.format(el))
+                log.trace('found available service path: %s', el)
 
     if status == 'DISABLED':
         # service available but not enabled
@@ -588,7 +594,7 @@ def enable(name, start=False, **kwargs):
             try:
                 os.unlink(down_file)
             except OSError:
-                log.error('Unable to remove file {0}'.format(down_file))
+                log.error('Unable to remove file %s', down_file)
                 return False
         return True
 
@@ -597,10 +603,10 @@ def enable(name, start=False, **kwargs):
     if not start:
         # create a temp 'down' file BEFORE enabling service.
         # will prevent sv from starting this service automatically.
-        log.trace('need a temporary file {0}'.format(down_file))
+        log.trace('need a temporary file %s', down_file)
         if not os.path.exists(down_file):
             try:
-                salt.utils.fopen(down_file, "w").close()  # pylint: disable=resource-leakage
+                salt.utils.files.fopen(down_file, "w").close()  # pylint: disable=resource-leakage
             except IOError:
                 log.error('Unable to create file {0}'.format(down_file))
                 return False
@@ -634,7 +640,7 @@ def enable(name, start=False, **kwargs):
         try:
             os.unlink(down_file)
         except OSError:
-            log.error('Unable to remove temp file {0}'.format(down_file))
+            log.error('Unable to remove temp file %s', down_file)
             retcode_sv = 1
 
     # if an error happened, revert our changes
@@ -675,9 +681,9 @@ def disable(name, stop=False, **kwargs):
 
     if not os.path.exists(down_file):
         try:
-            salt.utils.fopen(down_file, "w").close()  # pylint: disable=resource-leakage
+            salt.utils.files.fopen(down_file, "w").close()  # pylint: disable=resource-leakage
         except IOError:
-            log.error('Unable to create file {0}'.format(down_file))
+            log.error('Unable to create file %s', down_file)
             return False
 
     return True
@@ -704,16 +710,16 @@ def remove(name):
 
     svc_path = _service_path(name)
     if not os.path.islink(svc_path):
-        log.error('{0} is not a symlink: not removed'.format(svc_path))
+        log.error('%s is not a symlink: not removed', svc_path)
         return False
 
     if not stop(name):
-        log.error('Failed to stop service {0}'.format(name))
+        log.error('Failed to stop service %s', name)
         return False
     try:
         os.remove(svc_path)
     except IOError:
-        log.error('Unable to remove symlink {0}'.format(svc_path))
+        log.error('Unable to remove symlink %s', svc_path)
         return False
     return True
 
