@@ -1,6 +1,4 @@
 # encoding: utf-8
-from __future__ import absolute_import, print_function
-
 '''
 A non-blocking REST API for Salt
 ================================
@@ -186,8 +184,8 @@ a return like::
 .. |401| replace:: authentication required
 .. |406| replace:: requested Content-Type not available
 .. |500| replace:: internal server error
-'''  # pylint: disable=W0105
-# pylint: disable=W0232
+'''
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import Python libs
 import time
@@ -198,32 +196,45 @@ from collections import defaultdict
 
 # pylint: disable=import-error
 import cgi
-import yaml
 import tornado.escape
 import tornado.httpserver
 import tornado.ioloop
 import tornado.web
 import tornado.gen
 from tornado.concurrent import Future
-from zmq.eventloop import ioloop
-import salt.ext.six as six
 # pylint: enable=import-error
-
-# instantiate the zmq IOLoop (specialized poller)
-ioloop.install()
+import salt.utils
+salt.utils.zeromq.install_zmq()
 
 # salt imports
+import salt.ext.six as six
 import salt.netapi
-import salt.utils
+import salt.utils.args
 import salt.utils.event
+import salt.utils.jid
+import salt.utils.json
+import salt.utils.yaml
 from salt.utils.event import tagify
 import salt.client
 import salt.runner
 import salt.auth
-from salt.exceptions import EauthAuthenticationError
+from salt.exceptions import (
+    AuthenticationError,
+    AuthorizationError,
+    EauthAuthenticationError
+)
 
-json = salt.utils.import_json()
-logger = logging.getLogger()
+json = salt.utils.json.import_json()
+log = logging.getLogger(__name__)
+
+
+def _json_dumps(obj, **kwargs):
+    '''
+    Invoke salt.utils.json.dumps using the alternate json module loaded using
+    salt.utils.json.import_json(). This ensures that we properly encode any
+    strings in the object before we perform the serialization.
+    '''
+    return salt.utils.json.dumps(obj, _json_module=json, **kwargs)
 
 # The clients rest_cherrypi supports. We want to mimic the interface, but not
 #     necessarily use the same API under the hood
@@ -236,28 +247,6 @@ logger = logging.getLogger()
 #  - "wheel" (need async api...)
 
 
-class SaltClientsMixIn(object):
-    '''
-    MixIn class to container all of the salt clients that the API needs
-    '''
-    # TODO: load this proactively, instead of waiting for a request
-    __saltclients = None
-
-    @property
-    def saltclients(self):
-        if SaltClientsMixIn.__saltclients is None:
-            local_client = salt.client.get_local_client(mopts=self.application.opts)
-            # TODO: refreshing clients using cachedict
-            SaltClientsMixIn.__saltclients = {
-                'local': local_client.run_job_async,
-                # not the actual client we'll use.. but its what we'll use to get args
-                'local_async': local_client.run_job_async,
-                'runner': salt.runner.RunnerClient(opts=self.application.opts).cmd_async,
-                'runner_async': None,  # empty, since we use the same client as `runner`
-                }
-        return SaltClientsMixIn.__saltclients
-
-
 AUTH_TOKEN_HEADER = 'X-Auth-Token'
 AUTH_COOKIE_NAME = 'session_id'
 
@@ -311,15 +300,15 @@ class EventListener(object):
 
         self.event.set_event_handler(self._handle_event_socket_recv)
 
-    def clean_timeout_futures(self, request):
+    def clean_by_request(self, request):
         '''
         Remove all futures that were waiting for request `request` since it is done waiting
         '''
         if request not in self.request_map:
             return
-        for tag, future in self.request_map[request]:
+        for tag, matcher, future in self.request_map[request]:
             # timeout the future
-            self._timeout_future(tag, future)
+            self._timeout_future(tag, matcher, future)
             # remove the timeout
             if future in self.timeout_map:
                 tornado.ioloop.IOLoop.current().remove_timeout(self.timeout_map[future])
@@ -327,9 +316,22 @@ class EventListener(object):
 
         del self.request_map[request]
 
+    @staticmethod
+    def prefix_matcher(mtag, tag):
+        if mtag is None or tag is None:
+            raise TypeError('mtag or tag can not be None')
+        return mtag.startswith(tag)
+
+    @staticmethod
+    def exact_matcher(mtag, tag):
+        if mtag is None or tag is None:
+            raise TypeError('mtag or tag can not be None')
+        return mtag == tag
+
     def get_event(self,
                   request,
                   tag='',
+                  matcher=prefix_matcher.__func__,
                   callback=None,
                   timeout=None
                   ):
@@ -349,49 +351,58 @@ class EventListener(object):
                 tornado.ioloop.IOLoop.current().add_callback(callback, future)
             future.add_done_callback(handle_future)
         # add this tag and future to the callbacks
-        self.tag_map[tag].append(future)
-        self.request_map[request].append((tag, future))
+        self.tag_map[(tag, matcher)].append(future)
+        self.request_map[request].append((tag, matcher, future))
 
         if timeout:
-            timeout_future = tornado.ioloop.IOLoop.current().call_later(timeout, self._timeout_future, tag, future)
+            timeout_future = tornado.ioloop.IOLoop.current().call_later(timeout, self._timeout_future, tag, matcher, future)
             self.timeout_map[future] = timeout_future
 
         return future
 
-    def _timeout_future(self, tag, future):
+    def _timeout_future(self, tag, matcher, future):
         '''
         Timeout a specific future
         '''
-        if tag not in self.tag_map:
+        if (tag, matcher) not in self.tag_map:
             return
         if not future.done():
             future.set_exception(TimeoutException())
-            self.tag_map[tag].remove(future)
-        if len(self.tag_map[tag]) == 0:
-            del self.tag_map[tag]
+            self.tag_map[(tag, matcher)].remove(future)
+        if len(self.tag_map[(tag, matcher)]) == 0:
+            del self.tag_map[(tag, matcher)]
 
     def _handle_event_socket_recv(self, raw):
         '''
         Callback for events on the event sub socket
         '''
         mtag, data = self.event.unpack(raw, self.event.serial)
+
         # see if we have any futures that need this info:
-        for tag_prefix, futures in six.iteritems(self.tag_map):
-            if mtag.startswith(tag_prefix):
-                for future in futures:
-                    if future.done():
-                        continue
-                    future.set_result({'data': data, 'tag': mtag})
-                    self.tag_map[tag_prefix].remove(future)
-                    if future in self.timeout_map:
-                        tornado.ioloop.IOLoop.current().remove_timeout(self.timeout_map[future])
-                        del self.timeout_map[future]
+        for (tag, matcher), futures in six.iteritems(self.tag_map):
+            try:
+                is_matched = matcher(mtag, tag)
+            except Exception:
+                log.error('Failed to run a matcher.', exc_info=True)
+                is_matched = False
+
+            if not is_matched:
+                continue
+
+            for future in futures:
+                if future.done():
+                    continue
+                future.set_result({'data': data, 'tag': mtag})
+                self.tag_map[(tag, matcher)].remove(future)
+                if future in self.timeout_map:
+                    tornado.ioloop.IOLoop.current().remove_timeout(self.timeout_map[future])
+                    del self.timeout_map[future]
 
 
-class BaseSaltAPIHandler(tornado.web.RequestHandler, SaltClientsMixIn):  # pylint: disable=W0223
+class BaseSaltAPIHandler(tornado.web.RequestHandler):  # pylint: disable=W0223
     ct_out_map = (
-        ('application/json', json.dumps),
-        ('application/x-yaml', yaml.safe_dump),
+        ('application/json', _json_dumps),
+        ('application/x-yaml', salt.utils.yaml.safe_dump),
     )
 
     def _verify_client(self, low):
@@ -410,12 +421,25 @@ class BaseSaltAPIHandler(tornado.web.RequestHandler, SaltClientsMixIn):  # pylin
         Initialize the handler before requests are called
         '''
         if not hasattr(self.application, 'event_listener'):
-            logger.critical('init a listener')
+            log.debug('init a listener')
             self.application.event_listener = EventListener(
                 self.application.mod_opts,
                 self.application.opts,
             )
 
+        if not hasattr(self, 'saltclients'):
+            local_client = salt.client.get_local_client(mopts=self.application.opts)
+            self.saltclients = {
+                'local': local_client.run_job_async,
+                # not the actual client we'll use.. but its what we'll use to get args
+                'local_async': local_client.run_job_async,
+                'runner': salt.runner.RunnerClient(opts=self.application.opts).cmd_async,
+                'runner_async': None,  # empty, since we use the same client as `runner`
+                }
+
+        if not hasattr(self, 'ckminions'):
+            self.ckminions = salt.utils.minions.CkMinions(self.application.opts)
+
     @property
     def token(self):
         '''
@@ -471,7 +495,7 @@ class BaseSaltAPIHandler(tornado.web.RequestHandler, SaltClientsMixIn):  # pylin
         timeout a session
         '''
         # TODO: set a header or something??? so we know it was a timeout
-        self.application.event_listener.clean_timeout_futures(self)
+        self.application.event_listener.clean_by_request(self)
 
     def on_finish(self):
         '''
@@ -514,17 +538,16 @@ class BaseSaltAPIHandler(tornado.web.RequestHandler, SaltClientsMixIn):  # pylin
         '''
         ct_in_map = {
             'application/x-www-form-urlencoded': self._form_loader,
-            'application/json': json.loads,
-            'application/x-yaml': yaml.safe_load,
-            'text/yaml': yaml.safe_load,
+            'application/json': salt.utils.json.loads,
+            'application/x-yaml': salt.utils.yaml.safe_load,
+            'text/yaml': salt.utils.yaml.safe_load,
             # because people are terrible and don't mean what they say
-            'text/plain': json.loads
+            'text/plain': salt.utils.json.loads
         }
 
         try:
             # Use cgi.parse_header to correctly separate parameters from value
-            header = cgi.parse_header(self.request.headers['Content-Type'])
-            value, parameters = header
+            value, parameters = cgi.parse_header(self.request.headers['Content-Type'])
             return ct_in_map[value](tornado.escape.native_str(data))
         except KeyError:
             self.send_error(406)
@@ -538,7 +561,7 @@ class BaseSaltAPIHandler(tornado.web.RequestHandler, SaltClientsMixIn):  # pylin
         if not self.request.body:
             return
         data = self.deserialize(self.request.body)
-        self.raw_data = copy(data)
+        self.request_payload = copy(data)
 
         if data and 'arg' in data and not isinstance(data['arg'], list):
             data['arg'] = [data['arg']]
@@ -696,15 +719,13 @@ class SaltAuthHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
             }}
         '''
         try:
-            request_payload = self.deserialize(self.request.body)
-
-            if not isinstance(request_payload, dict):
+            if not isinstance(self.request_payload, dict):
                 self.send_error(400)
                 return
 
-            creds = {'username': request_payload['username'],
-                     'password': request_payload['password'],
-                     'eauth': request_payload['eauth'],
+            creds = {'username': self.request_payload['username'],
+                     'password': self.request_payload['password'],
+                     'eauth': self.request_payload['eauth'],
                      }
         # if any of the args are missing, its a bad request
         except KeyError:
@@ -729,9 +750,11 @@ class SaltAuthHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
             return
 
         except (AttributeError, IndexError):
-            logging.debug("Configuration for external_auth malformed for "
-                          "eauth '{0}', and user '{1}'."
-                          .format(token.get('eauth'), token.get('name')), exc_info=True)
+            log.debug(
+                "Configuration for external_auth malformed for eauth '%s', "
+                "and user '%s'.", token.get('eauth'), token.get('name'),
+                exc_info=True
+            )
             # TODO better error -- 'Configuration for external_auth could not be read.'
             self.send_error(500)
             return
@@ -748,7 +771,7 @@ class SaltAuthHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
         self.write(self.serialize(ret))
 
 
-class SaltAPIHandler(BaseSaltAPIHandler, SaltClientsMixIn):  # pylint: disable=W0223
+class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
     '''
     Main API handler for base "/"
     '''
@@ -892,12 +915,12 @@ class SaltAPIHandler(BaseSaltAPIHandler, SaltClientsMixIn):  # pylint: disable=W
             try:
                 chunk_ret = yield getattr(self, '_disbatch_{0}'.format(low['client']))(low)
                 ret.append(chunk_ret)
-            except EauthAuthenticationError as exc:
+            except (AuthenticationError, AuthorizationError, EauthAuthenticationError):
                 ret.append('Failed to authenticate')
                 break
             except Exception as ex:
                 ret.append('Unexpected exception while handling request: {0}'.format(ex))
-                logger.error('Unexpected exception while handling request:', exc_info=True)
+                log.error('Unexpected exception while handling request:', exc_info=True)
 
         self.write(self.serialize({'return': ret}))
         self.finish()
@@ -907,92 +930,106 @@ class SaltAPIHandler(BaseSaltAPIHandler, SaltClientsMixIn):  # pylint: disable=W
         '''
         Dispatch local client commands
         '''
-        chunk_ret = {}
+        # Generate jid before triggering a job to subscribe all returns from minions
+        chunk['jid'] = salt.utils.jid.gen_jid(self.application.opts)
+
+        # Subscribe returns from minions before firing a job
+        minions = set(self.ckminions.check_minions(chunk['tgt'], chunk.get('tgt_type', 'glob')))
+        future_minion_map = self.subscribe_minion_returns(chunk['jid'], minions)
 
         f_call = self._format_call_run_job_async(chunk)
         # fire a job off
-        try:
-            pub_data = yield self.saltclients['local'](*f_call.get('args', ()), **f_call.get('kwargs', {}))
-        except EauthAuthenticationError:
-            raise tornado.gen.Return('Not authorized to run this job')
+        pub_data = yield self.saltclients['local'](*f_call.get('args', ()), **f_call.get('kwargs', {}))
 
         # if the job didn't publish, lets not wait around for nothing
         # TODO: set header??
         if 'jid' not in pub_data:
+            for future in future_minion_map:
+                try:
+                    future.set_result(None)
+                except Exception:
+                    pass
             raise tornado.gen.Return('No minions matched the target. No command was sent, no jid was assigned.')
 
-        # seed minions_remaining with the pub_data
-        minions_remaining = pub_data['minions']
-
-        syndic_min_wait = None
+        # wait syndic a while to avoid missing published events
         if self.application.opts['order_masters']:
-            syndic_min_wait = tornado.gen.sleep(self.application.opts['syndic_wait'])
+            yield tornado.gen.sleep(self.application.opts['syndic_wait'])
 
-        job_not_running = self.job_not_running(pub_data['jid'],
-                                               chunk['tgt'],
-                                               f_call['kwargs']['tgt_type'],
-                                               minions_remaining=minions_remaining
-                                               )
+        # To ensure job_not_running and all_return are terminated by each other, communicate using a future
+        is_finished = Future()
 
-        # if we have a min_wait, do that
-        if syndic_min_wait is not None:
-            yield syndic_min_wait
-        # we are completed when either all minions return or the job isn't running anywhere
-        chunk_ret = yield self.all_returns(pub_data['jid'],
-                                           finish_futures=[job_not_running],
-                                           minions_remaining=minions_remaining,
-                                           )
+        job_not_running_future = self.job_not_running(pub_data['jid'],
+                                                      chunk['tgt'],
+                                                      f_call['kwargs']['tgt_type'],
+                                                      is_finished)
 
-        raise tornado.gen.Return(chunk_ret)
+        minion_returns_future = self.sanitize_minion_returns(future_minion_map, pub_data['minions'], is_finished)
+
+        yield job_not_running_future
+        raise tornado.gen.Return((yield minion_returns_future))
+
+    def subscribe_minion_returns(self, jid, minions):
+        # Subscribe each minion event
+        future_minion_map = {}
+        for minion in minions:
+            tag = tagify([jid, 'ret', minion], 'job')
+            minion_future = self.application.event_listener.get_event(self,
+                                                                      tag=tag,
+                                                                      matcher=EventListener.exact_matcher)
+            future_minion_map[minion_future] = minion
+        return future_minion_map
 
     @tornado.gen.coroutine
-    def all_returns(self,
-                    jid,
-                    finish_futures=None,
-                    minions_remaining=None,
-                    ):
+    def sanitize_minion_returns(self, future_minion_map, minions, is_finished):
         '''
         Return a future which will complete once all returns are completed
-        (according to minions_remaining), or one of the passed in "finish_futures" completes
+        (according to minions), or one of the passed in "finish_chunk_ret_future" completes
         '''
-        if finish_futures is None:
-            finish_futures = []
-        if minions_remaining is None:
-            minions_remaining = []
+        if minions is None:
+            minions = []
+
+        # Remove redundant minions
+        redundant_minion_futures = [future for future in future_minion_map.keys() if future_minion_map[future] not in minions]
+        for redundant_minion_future in redundant_minion_futures:
+            try:
+                redundant_minion_future.set_result(None)
+            except Exception:
+                pass
+            del future_minion_map[redundant_minion_future]
 
-        ret_tag = tagify([jid, 'ret'], 'job')
         chunk_ret = {}
         while True:
-            ret_event = self.application.event_listener.get_event(self,
-                                                      tag=ret_tag,
-                                                      )
-            f = yield Any([ret_event] + finish_futures)
-            if f in finish_futures:
-                raise tornado.gen.Return(chunk_ret)
-            event = f.result()
-            chunk_ret[event['data']['id']] = event['data']['return']
-            # its possible to get a return that wasn't in the minion_remaining list
+            f = yield Any(list(future_minion_map.keys()) + [is_finished])
             try:
-                minions_remaining.remove(event['data']['id'])
+                # When finished entire routine, cleanup other futures and return result
+                if f is is_finished:
+                    for event in future_minion_map.keys():
+                        if not event.done():
+                            event.set_result(None)
+                    raise tornado.gen.Return(chunk_ret)
+                f_result = f.result()
+                chunk_ret[f_result['data']['id']] = f_result['data']['return']
+            except TimeoutException:
+                pass
+
+            # clear finished event future
+            try:
+                minions.remove(future_minion_map[f])
+                del future_minion_map[f]
             except ValueError:
                 pass
-            if len(minions_remaining) == 0:
+
+            if not minions:
+                if not is_finished.done():
+                    is_finished.set_result(True)
                 raise tornado.gen.Return(chunk_ret)
 
     @tornado.gen.coroutine
-    def job_not_running(self,
-                  jid,
-                  tgt,
-                  tgt_type,
-                  minions_remaining=None,
-                  ):
+    def job_not_running(self, jid, tgt, tgt_type, is_finished):
         '''
         Return a future which will complete once jid (passed in) is no longer
         running on tgt
         '''
-        if minions_remaining is None:
-            minions_remaining = []
-
         ping_pub_data = yield self.saltclients['local'](tgt,
                                                         'saltutil.find_job',
                                                         [jid],
@@ -1002,12 +1039,20 @@ class SaltAPIHandler(BaseSaltAPIHandler, SaltClientsMixIn):  # pylint: disable=W
         minion_running = False
         while True:
             try:
-                event = yield self.application.event_listener.get_event(self,
-                                                                        tag=ping_tag,
-                                                                        timeout=self.application.opts['gather_job_timeout'],
-                                                                        )
+                event = self.application.event_listener.get_event(self,
+                                                                  tag=ping_tag,
+                                                                  timeout=self.application.opts['gather_job_timeout'])
+                f = yield Any([event, is_finished])
+                # When finished entire routine, cleanup other futures and return result
+                if f is is_finished:
+                    if not event.done():
+                        event.set_result(None)
+                    raise tornado.gen.Return(True)
+                event = f.result()
             except TimeoutException:
                 if not minion_running:
+                    if not is_finished.done():
+                        is_finished.set_result(True)
                     raise tornado.gen.Return(True)
                 else:
                     ping_pub_data = yield self.saltclients['local'](tgt,
@@ -1017,13 +1062,11 @@ class SaltAPIHandler(BaseSaltAPIHandler, SaltClientsMixIn):  # pylint: disable=W
                     ping_tag = tagify([ping_pub_data['jid'], 'ret'], 'job')
                     minion_running = False
                     continue
+
             # Minions can return, we want to see if the job is running...
             if event['data'].get('return', {}) == {}:
                 continue
             minion_running = True
-            id_ = event['data']['id']
-            if id_ not in minions_remaining:
-                minions_remaining.append(event['data']['id'])
 
     @tornado.gen.coroutine
     def _disbatch_local_async(self, chunk):
@@ -1059,9 +1102,13 @@ class SaltAPIHandler(BaseSaltAPIHandler, SaltClientsMixIn):  # pylint: disable=W
         pub_data = self.saltclients['runner'](chunk)
         raise tornado.gen.Return(pub_data)
 
-    # salt.utils.format_call doesn't work for functions having the annotation tornado.gen.coroutine
+    # salt.utils.args.format_call doesn't work for functions having the
+    # annotation tornado.gen.coroutine
     def _format_call_run_job_async(self, chunk):
-        f_call = salt.utils.format_call(salt.client.LocalClient.run_job, chunk, is_class_method=True)
+        f_call = salt.utils.args.format_call(
+            salt.client.LocalClient.run_job,
+            chunk,
+            is_class_method=True)
         f_call.get('kwargs', {})['io_loop'] = tornado.ioloop.IOLoop.current()
         return f_call
 
@@ -1480,14 +1527,14 @@ class EventsSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
         self.set_header('Cache-Control', 'no-cache')
         self.set_header('Connection', 'keep-alive')
 
-        self.write(u'retry: {0}\n'.format(400))
+        self.write('retry: {0}\n'.format(400))
         self.flush()
 
         while True:
             try:
                 event = yield self.application.event_listener.get_event(self)
-                self.write(u'tag: {0}\n'.format(event.get('tag', '')))
-                self.write(u'data: {0}\n\n'.format(json.dumps(event)))
+                self.write('tag: {0}\n'.format(event.get('tag', '')))
+                self.write(str('data: {0}\n\n').format(_json_dumps(event)))  # future lint: disable=blacklisted-function
                 self.flush()
             except TimeoutException:
                 break
@@ -1641,7 +1688,7 @@ class WebhookSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
                 value = value[0]
             arguments[argname] = value
         ret = self.event.fire_event({
-            'post': self.raw_data,
+            'post': self.request_payload,
             'get': arguments,
             # In Tornado >= v4.0.3, the headers come
             # back as an HTTPHeaders instance, which
