@@ -4,7 +4,7 @@ This module contains the function calls to execute command line scripts
 '''
 
 # Import python libs
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 import sys
 import time
@@ -13,6 +13,8 @@ import logging
 import functools
 import threading
 import traceback
+import signal
+import functools
 from random import randint
 
 # Import salt libs
@@ -41,7 +43,12 @@ def _handle_interrupt(exc, original_exc, hardfail=False, trace=''):
 
 
 def _handle_signals(client, signum, sigframe):
-    trace = traceback.format_exc()
+    try:
+        # This raises AttributeError on Python 3.4 and 3.5 if there is no current exception.
+        # Ref: https://bugs.python.org/issue23003
+        trace = traceback.format_exc()
+    except AttributeError:
+        trace = ''
     try:
         hardcrash = client.options.hard_crash
     except (AttributeError, KeyError):
@@ -94,11 +101,16 @@ def minion_process():
     '''
     Start a minion process
     '''
-    import salt.utils
+    import salt.utils.platform
+    import salt.utils.process
     import salt.cli.daemons
+
     # salt_minion spawns this function in a new process
 
-    salt.utils.appendproctitle('KeepAlive')
+    salt.utils.process.appendproctitle('KeepAlive')
+
+    def handle_hup(manager, sig, frame):
+        manager.minion.reload()
 
     def suicide_when_without_parent(parent_pid):
         '''
@@ -111,19 +123,22 @@ def minion_process():
             time.sleep(5)
             try:
                 # check pid alive (Unix only trick!)
-                if os.getuid() == 0 and not salt.utils.is_windows():
+                if os.getuid() == 0 and not salt.utils.platform.is_windows():
                     os.kill(parent_pid, 0)
             except OSError as exc:
                 # forcibly exit, regular sys.exit raises an exception-- which
                 # isn't sufficient in a thread
-                log.error('Minion process encountered exception: {0}'.format(exc))
+                log.error('Minion process encountered exception: %s', exc)
                 os._exit(salt.defaults.exitcodes.EX_GENERIC)
 
-    if not salt.utils.is_windows():
+    if not salt.utils.platform.is_windows():
         thread = threading.Thread(target=suicide_when_without_parent, args=(os.getppid(),))
         thread.start()
 
     minion = salt.cli.daemons.Minion()
+    signal.signal(signal.SIGHUP,
+                  functools.partial(handle_hup,
+                                    minion))
 
     try:
         minion.start()
@@ -134,7 +149,7 @@ def minion_process():
         if minion is not None and hasattr(minion, 'config'):
             delay = minion.config.get('random_reauth_delay', 60)
         delay = randint(1, delay)
-        log.info('waiting random_reauth_delay {0}s'.format(delay))
+        log.info('waiting random_reauth_delay %ss', delay)
         time.sleep(delay)
         sys.exit(salt.defaults.exitcodes.SALT_KEEPALIVE)
 
@@ -146,6 +161,7 @@ def salt_minion():
     '''
     import signal
 
+    import salt.utils.platform
     import salt.utils.process
     salt.utils.process.notify_systemd()
 
@@ -154,7 +170,7 @@ def salt_minion():
     if '' in sys.path:
         sys.path.remove('')
 
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         minion = salt.cli.daemons.Minion()
         minion.start()
         return
@@ -186,6 +202,9 @@ def salt_minion():
             signal.signal(signal.SIGINT,
                           functools.partial(escalate_signal_to_process,
                                             process.pid))
+            signal.signal(signal.SIGHUP,
+                          functools.partial(escalate_signal_to_process,
+                                            process.pid))
         except Exception:  # pylint: disable=broad-except
             # if multiprocessing does not work
             minion = salt.cli.daemons.Minion()
@@ -218,6 +237,7 @@ def proxy_minion_process(queue):
     Start a proxy minion process
     '''
     import salt.cli.daemons
+    import salt.utils.platform
     # salt_minion spawns this function in a new process
 
     def suicide_when_without_parent(parent_pid):
@@ -237,7 +257,7 @@ def proxy_minion_process(queue):
                 # isn't sufficient in a thread
                 os._exit(999)
 
-    if not salt.utils.is_windows():
+    if not salt.utils.platform.is_windows():
         thread = threading.Thread(target=suicide_when_without_parent, args=(os.getppid(),))
         thread.start()
 
@@ -263,7 +283,7 @@ def proxy_minion_process(queue):
             if hasattr(proxyminion, 'config'):
                 delay = proxyminion.config.get('random_reauth_delay', 60)
         random_delay = randint(1, delay)
-        log.info('Sleeping random_reauth_delay of {0} seconds'.format(random_delay))
+        log.info('Sleeping random_reauth_delay of %s seconds', random_delay)
         # preform delay after minion resources have been cleaned
         queue.put(random_delay)
     else:
@@ -271,16 +291,17 @@ def proxy_minion_process(queue):
     sys.exit(status)
 
 
-def salt_proxy_minion():
+def salt_proxy():
     '''
     Start a proxy minion.
     '''
     import salt.cli.daemons
+    import salt.utils.platform
     import multiprocessing
     if '' in sys.path:
         sys.path.remove('')
 
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         proxyminion = salt.cli.daemons.ProxyMinion()
         proxyminion.start()
         return
@@ -418,21 +439,21 @@ def salt_cloud():
     '''
     The main function for salt-cloud
     '''
+    # Define 'salt' global so we may use it after ImportError. Otherwise,
+    # UnboundLocalError will be raised.
+    global salt  # pylint: disable=W0602
+
     try:
         # Late-imports for CLI performance
         import salt.cloud
         import salt.cloud.cli
-        has_saltcloud = True
     except ImportError as e:
-        log.error("Error importing salt cloud {0}".format(e))
         # No salt cloud on Windows
-        has_saltcloud = False
-    if '' in sys.path:
-        sys.path.remove('')
-
-    if not has_saltcloud:
+        log.error('Error importing salt cloud: %s', e)
         print('salt-cloud is not available in this system')
         sys.exit(salt.defaults.exitcodes.EX_UNAVAILABLE)
+    if '' in sys.path:
+        sys.path.remove('')
 
     client = salt.cloud.cli.SaltCloud()
     _install_signal_handlers(client)
